/*
 * Copyright (c) 2004 Stefan Ulrich
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

/*
 * Preferences dialog for xdvik.

 TODO:

 - update texmf/XDvi

 - implement callbacks for:
    . Text color/background
    . String matches
    . Show Tooltips
    . Helper Applications

 - implement Paper Size panel   
 
 - copy preferences to G_preferences and use G_preferences.xyz everywhere
   instead of resource.xyz
   
 - implement $HOME/.xdvirc file reading and writing (using Xrm* functions from xlib?
   see ch. 15 of the Xlib Programming Manual; and:
   http://groups.google.com/groups?selm=1994Dec31.162840.11637%40zh014.ubs.ubs.ch

   Maybe the best way to do this is to update the resource manager whenever one
   of the preferences options changes, and then get the resource manager values.

 - rename xm_prefs?.c to xm_prefs_app.c etc.

 - set text `with ___ milliseconds delay' to insensitive

 - add highlight/border colors to `Fonts and Colors'

 - don't change GC's or update the display until color in `Fonts and Colors'
   is actually changed (currently it changes also when e.g. selecting the
   panel)
 
   
 */

#include "xdvi-config.h"
#include "xdvi.h"

#include "x_util.h"
#include "xm_colorsel.h"
#include "topic-window.h"
#include "util.h"
#include "events.h"

#include "xm_prefsP.h"
#include "xm_prefs.h"

#include "xm_prefs1.h" /* panel 1 */
#include "xm_prefs2.h" /* panel 2 */
#include "xm_prefs3.h" /* panel 3 */
#include "xm_prefs4.h" /* panel 4 */

#ifdef MOTIF /* entire file */

#include <X11/Xatom.h>

#include <Xm/Xm.h>
#include <Xm/Protocols.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>

#define NUM_PREFS_TOPICS 4

/* hmm, should maybe move these into xm_prefsP.c ... */
void
h_attach_below(Widget x, Widget y) {
    if (y == NULL) {
	/* no top widget, attach to form - could just pass NULL as XmNtopWidget, but lesstif warns in that case ... */
	XtVaSetValues(x,
		  XmNtopAttachment, XmATTACH_FORM,
		  XmNleftAttachment, XmATTACH_FORM,
		  XmNrightAttachment, XmATTACH_FORM,
		  NULL);
    }
    else {
	XtVaSetValues(x,
		      XmNtopAttachment, XmATTACH_WIDGET,
		      XmNtopWidget, y,
		      XmNleftAttachment, XmATTACH_FORM,
		  XmNrightAttachment, XmATTACH_FORM,
		      NULL);
    }
}


static void
initialize_items(struct topic_info *info)
{

    info->items[0].widget = prefs_appearance(info);
    info->items[0].topic = xstrdup("Appearance");
    info->items[0].title = xstrdup("Change the Appearance of Xdvi");
    
    info->items[1].widget = prefs_fonts_and_colors(info);
    info->items[1].topic = xstrdup("Fonts and Colors");
    info->items[1].title = xstrdup("Display of Fonts and Colors in DVI files");

    info->items[2].widget = prefs_helpers(info);
    info->items[2].topic = xstrdup("Helper Applications");
    info->items[2].title = xstrdup("External Programs used by Xdvi");

    info->items[3].widget = prefs_paper(info);
    info->items[3].topic = xstrdup("Paper Size");
    info->items[3].title = xstrdup("Set Default Paper Size and Offsets");
}

void
remove_from_deplist(struct prefs_choice *prefs, Widget w)
{
    size_t i;
    Boolean found = False;

    fprintf(stderr, "removing from deplist: %p\n", w);
    /* locate this widget */
    for (i = 0; i < prefs->depwin_cnt; i++) {
	if (prefs->depwin[i] == w) {
	    fprintf(stderr, "found window at place %d in deplist\n", i);
	    found = True;
	    break;
	}
    }
    
    if (found) {
	/* move later widgets down */
	for (; i < prefs->depwin_cnt - 1; i++) {
	    prefs->depwin[i] = prefs->depwin[i + 1];
	}
	prefs->depwin_cnt--;
    }

#if 0
    fprintf(stderr, "deplist after removal:\n");
    for (i = 0; i < prefs->depwin_cnt; i++) {
	fprintf(stderr, "%d: %p\n", i, prefs->depwin[i]);
    }
#endif    
}

/* add widget w to list of dependent windows */
void
add_to_deplist(struct prefs_choice *prefs, Widget w)
{
    size_t i;
    fprintf(stderr, "adding to deplist: %p\n", w);
    
    prefs->depwin = xrealloc(prefs->depwin,
			     (prefs->depwin_cnt + 1) * sizeof *(prefs->depwin));
    prefs->depwin[prefs->depwin_cnt] = w;
    prefs->depwin_cnt++;

    fprintf(stderr, "deplist after adding:\n");
    for (i = 0; i < prefs->depwin_cnt; i++) {
	fprintf(stderr, "%d: %p\n", i, prefs->depwin[i]);
    }
}

static void
set_prefs_cb(XtPointer arg)
{
    struct topic_info *info = (struct topic_info *)arg;
    struct prefs_choice *prefs = (struct prefs_choice *)info->data;
    size_t i;
    Widget colorsel;

    if (get_widget_by_name(&colorsel, top_level, Xdvi_COLOR_DIALOG_NAME, False)) {
	XtPopdown(XtParent(colorsel));
    }

    /* pop down dependent windows */
    TRACE_GUI((stderr, "window count: %d\n", prefs->depwin_cnt));
    for (i = 0; i < prefs->depwin_cnt; i++) {
	TRACE_GUI((stderr, "popping down %d: %p", i, prefs->depwin[i]));
	if (XtIsRealized(prefs->depwin[i]))
	    XtCallCallbacks(prefs->depwin[i], XmNcancelCallback, NULL);
    }
    free(prefs->depwin);
    prefs->depwin = NULL;
    prefs->depwin_cnt = 0;

    /*XXX FIXME: needs re-initialization when opening prefs window! */
    if (prefs->changed == NULL) /* callback invoked multiple times? */
	return;
    
    copy_resources(prefs->changed, prefs->orig);
    free_resources(prefs->changed);
    prefs->changed = NULL;   
}

static void
revert_prefs_cb(XtPointer arg)
{
    struct topic_info *info = (struct topic_info *)arg;
    struct prefs_choice *prefs = (struct prefs_choice *)info->data;
    size_t i;

    /* pop down dependent windows */
    TRACE_GUI((stderr, "window count: %d", prefs->depwin_cnt));
    for (i = 0; i < prefs->depwin_cnt; i++) {
	TRACE_GUI((stderr, "popping down %d: %p", i, prefs->depwin[i]));
	if (XtIsManaged(prefs->depwin[i])) {
	    fprintf(stderr, "calling callback!\n");
	    XtCallCallbacks(prefs->depwin[i], XmNcancelCallback, NULL);
	}
	TRACE_GUI((stderr, "after popping down; count: %d, %d\n", i, prefs->depwin_cnt));
    }
    free(prefs->depwin);
    prefs->depwin = NULL;
    prefs->depwin_cnt = 0;

    /*XXX FIXME: needs re-initialization when opening prefs window! */
    if (prefs->changed == NULL) /* callback invoked multiple times? */
	return;

    free_resources(prefs->changed);
    prefs->changed = NULL;
}

void
popup_preferences_dialog(Widget parent, struct x_resources *orig_prefs)
{
    static Widget preferences_shell = 0;
    static struct topic_info info;
    static struct topic_item items[NUM_PREFS_TOPICS];
    static struct prefs_choice *prefs = NULL;

    if (preferences_shell == 0) { /* called 1st time; create widget */
	info.ok_callback = set_prefs_cb;
	info.cancel_callback = revert_prefs_cb;
	info.items = items;
	info.items_size = NUM_PREFS_TOPICS;

	prefs = xmalloc(sizeof *prefs);
	prefs->depwin_cnt = 0;
	prefs->depwin = NULL;
	prefs->orig = orig_prefs;
	/* set_prefs_cb/revert_prefs_cb are responsible for copying
	   the changed preferences into the current preferences as
	   appropriate, and free()ing prefs.changed */
	prefs->changed = xmalloc(sizeof *(prefs->changed));
	copy_resources(orig_prefs, prefs->changed);
	info.data = prefs;
	
	preferences_shell = create_topic_window(parent,
						"xdvik: Preferences",
						"preferences_window",
						&info,
						initialize_items,
						"OK", "Cancel");
	info.shell = preferences_shell;
	center_window(preferences_shell, parent);
	select_topic(&info, 2);
    }

    /*XXX TODO: initialize settings to orig_prefs!!! */
    XtPopup(preferences_shell, XtGrabNone);
}

#endif /* MOTIF */
